Object subclass: #EventNotification
	instanceVariableNames: 'block'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GUI-Utility'!

Object subclass: #EventSource
	instanceVariableNames: 'notifications source '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GUI-Utility'!

Object subclass: #NamedEventsSource
	instanceVariableNames: 'events source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GUI-Utility'!

!GTK.GUI.EventNotification methodsFor: 'initialization'!

activationBlock: aBlock
	aBlock numArgs > 1 ifTrue: 
		[ self error: 'Activation block can''t take more than one argument' ].
	block := aBlock
!

activationMsg: aSelector to: aReceiver from: aSource with: aData
	self activationBlock: 
		[ :data | self 
			send: aSelector
			to: aReceiver
			from: aSource
			with: aData
			with: data ].
! !

GTK.GUI.EventNotification methodsFor: 'activation'!

send: aSelector to: aReceiver from: aSource with: aFixedData with: aVariableData
    aSelector numArgs = 0 
		ifTrue: [ aReceiver perform: aSelector. ]
		ifFalse: 
			[ | args | 
			args := OrderedCollection with: aSource.
			aVariableData notNil ifTrue: [ args add: aVariableData ].
			aFixedData notNil ifTrue: [ args add: aFixedData ].
			aReceiver perform: aSelector withArguments: (args asArray). ].
!

activateWith: aVariableData
	block numArgs = 0
		ifTrue: [ block value ]
		ifFalse: [ block value: aVariableData ].
!

activate
	block numArgs = 0 
		ifTrue: [ block value ]
		ifFalse: [ block value: nil ].
!
!

!GTK.GUI.EventSource class methodsFor: 'instance creation'!

new
    | es |
    es := self basicNew.
    es source: es.
    ^es.
!

newFrom: aSource
    ^(self basicNew) source: aSource; yourself.
!
!

!GTK.GUI.EventSource methodsFor: 'initialization'!

source: aSource
    source := aSource
!
!

GTK.GUI.EventSource methodsFor: 'accessing'!

dontNotify: aReceiver
    notifications isNil ifTrue: [ ^self ].
    notifications removeKey: aReceiver ifAbsent: [].
!

send: aSelector to: aReceiver
    self send: aSelector to: aReceiver with: nil.
!
	
send: aSelector to: aReceiver with: aData
    self notifications 
		at: aReceiver 
		put: 
			((EventNotification new)
				activationMsg: aSelector to: aReceiver from: source with: aData;
				yourself).
!

update: aReceiver with: aBlock
	"aReceiver is nominal, since aBlock does not necessarily affect it."
    self notifications 
		at: aReceiver 
		put: 
			((EventNotification new) activationBlock: aBlock yourself).
!

updateWith: aBlock
	self update: Time now with: aBlock.
!
!

GTK.GUI.EventSource methodsFor: 'private'!

notifications
    notifications isNil ifTrue: [ notifications := WeakKeyDictionary new ].
	^notifications
!
!

!GTK.GUI.EventSource methodsFor: 'triggering'!

activate
    notifications isNil ifTrue: [ ^self ].
    notifications keysAndValuesDo: [ :key :val | val activate ].
!

activateWith: aVariableData
    notifications isNil ifTrue: [ ^self ].
    notifications keysAndValuesDo: [ :key :val | val activateWith: aVariableData ].
! 

!GTK.GUI.EventSource methodsFor: 'testing'!

hasListener
    ^notifications notNil
!    

hasHandler
    ^self hasListener
!
!

GTK.GUI.NamedEventsSource class methodsFor: 'instance creation'!

new
	^(super new) initialize; yourself.
!
!

GTK.GUI.NamedEventsSource methodsFor: 'initialization'!

initialize
    events := LookupTable new.
!

source: aSource
    source := aSource
!

add: aName
    events at: aName put: ((EventSource new) source: source; yourself).
!
!

GTK.GUI.NamedEventsSource methodsFor: 'accessing'!

includes: aEvent
    ^events includesKey: aEvent.
!

named: aName
    ^events at: aName
!

named: aName ifPresent: aBlock
    ^events at: aName ifPresent: [ :es | aBlock value: es ].
!

named: aName ifAbsent: aBlock
    ^events at: aName ifAbsent: [ aBlock value ].
!

on: aEvent dontNotify: aReceiver
    (self at: aEvent ifAbsent: [ ^self ])
		dontNotify: aReceiver.
!

on: aEvent send: aSelector to: aReceiver
    self on: aEvent send: aSelector to: aReceiver with: nil.
!

on: aEvent send: aSelector to: aReceiver with: aData
    (aReceiver respondsTo: aSelector) ifFalse: 
		[ self error: aReceiver printString, ' does not understand ', aSelector, ' - event ''', aEvent, ''' will not be connected.' ].
    (self named: aEvent) send: aSelector to: aReceiver with: aData
!

on: aEvent update: aReceiver with: aBlock
	(self named: aEvent) update: aReceiver with: aBlock
!

on: aEvent updateWith: aBlock
	(self named: aEvent) updateWith: aBlock
!

activate: aEvent
	self named: aEvent ifPresent: [ :es | es activate ].
!

activate: aEvent with: aData
	self named: aEvent ifPresent: [ :es | es activateWith: aData ].
!
!
