"======================================================================
|
|   Smalltalk GUI base class for widget wrappers with publish/subscribe
|
|
 ======================================================================"


"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson.
|
| This file is part of GNU Smalltalk.
|
| GNU Smalltalk is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| GNU Smalltalk; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"
	
Object subclass: #MenuItemList
	instanceVariableNames: 'items'
	classVariableNames: 'categoryWeights'
	poolDictionaries: ''
	category: 'GUI-Framework'!

GTK.GUI.MenuItemList class methodsFor: 'initialization'!

new
    ^(self basicNew) initialize.
!
!

GTK.GUI.MenuItemList methodsFor: 'initialization'!

initialize
    items := Dictionary new.
    ^self
!
!

GTK.GUI.MenuItemList methodsFor: 'accessing'!

add: aMenuItemView
    | cat |
	"super add: aMenuItemView."
    cat := items at: aMenuItemView category ifAbsentPut: [ OrderedCollection new ].
    cat add: aMenuItemView.
!

addAll: aMenuItemList
    aMenuItemList isNil ifTrue: [ ^self ].
    aMenuItemList viewsDo: [ :each | self add: each. ].
!

size
	^items size
!

categories
    ^items keys
!

categoryWeights
    categoryWeights isNil ifTrue: [
	categoryWeights := Dictionary new.
	categoryWeights 
	    at: 'Searching' put: 20;
	    at: 'Editing' put: 30;
	    at: 'Templates' put: 40;
	    at: 'Default' put: 50;
		at: 'Viewing' put: 65;
	    at: 'Filing' put: 80;
		at: 'Windows' put: 85;
	    at: 'Closing' put: 90;
	    at: 'Quitting' put: 95.
    ].
    ^categoryWeights.
!

categoryWeight: aCategory
    ^self categoryWeights 
	at: aCategory
	ifAbsentPut: [ 
	    categoryWeights at: 'Default' ifAbsent: [ 50 ] 
	].
!

byCategory
    ^(items keys asSortedCollection: 
		[ :a :b | 
		(self categoryWeight: a) <= (self categoryWeight: b)
		]) 
		collect: 
			[ :each | items at: each ].
!

viewsDo: aBlock
    items isNil ifTrue: [ ^self ].
    items do: [ :cat | cat do: aBlock ].
!
!

GTK.GUI.MenuItemList methodsFor: 'convenience'!

section: aSection add: aLabel sending: aSelector to: aReceiver
    "Convenience"
    self add: (MenuItemView category: aSection label: aLabel selector: aSelector receiver: aReceiver).
!
!
