"======================================================================
|
|   Smalltalk GUI base class for widget wrappers with publish/subscribe
|
|
 ======================================================================"

"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

GTK.GUI.View subclass: #MenuItemListView
	instanceVariableNames: 'items'
	classVariableNames: 'categoryWeights'
	poolDictionaries: ''
	category: 'GUI-Framework'!

GTK.GUI.Primitive subclass: #MenuItemView
	instanceVariableNames: 'category label selector receiver wgt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GUI-Framework'!

GTK.GUI.MenuItemListView class methodsFor: 'initialization'!

new
    ^(self basicNew) initialize.
!
!

GTK.GUI.MenuItemListView methodsFor: 'initialization'!

initialize
    items := Dictionary new.
    ^self
!
!

GTK.GUI.MenuItemListView methodsFor: 'accessing'!

add: aMenuItemView
    | cat |
	self add: aMenuItemView.
    cat := items at: aMenuItemView category ifAbsentPut: [ OrderedCollection new ].
    cat add: aMenuItemView.
!

addAll: aMenuItemListView
    aMenuItemListView isNil ifTrue: [ ^self ].
    aMenuItemListView viewsDo: [ :each | self add: each. ].
!

categories
    ^items keys
!

categoryWeights
    categoryWeights isNil ifTrue: [
	categoryWeights := Dictionary new.
	categoryWeights 
	    at: 'Searching' put: 20;
	    at: 'Editing' put: 30;
	    at: 'Templates' put: 40;
	    at: 'Default' put: 50;
	    at: 'Filing' put: 85;
	    at: 'Closing' put: 90;
	    at: 'Quitting' put: 95.
    ].
    ^categoryWeights.
!

categoryWeight: aCategory
    ^self categoryWeights 
	at: aCategory
	ifAbsentPut: [ 
	    categoryWeights at: 'Default' ifAbsent: [ 50 ] 
	].
!

byCategory
    ^(items keys asSortedCollection: [ :a :b | 
	(self categoryWeight: a) <= (self categoryWeight: b)
    ]) collect: [ :each | items at: each ].
!

viewsDo: aBlock
    items isNil ifTrue: [ ^self ].
    items do: [ :cat | cat do: aBlock ].
!
!

GTK.GUI.MenuItemListView methodsFor: 'convenience'!

section: aSection add: aLabel sending: aSelector to: aReceiver
    "Convenience"
    self add: (MenuItemView category: aSection label: aLabel selector: aSelector receiver: aReceiver).
!
!

GTK.GUI.MenuItemView class methodsFor: 'instance creation'!

category: aCategory label: aLabel selector: aSelector receiver: aReceiver
    ^(self basicNew) 
		category: aCategory label: aLabel selector: aSelector receiver: aReceiver;
		buildWidgets;
		yourself.
!
!

GTK.GUI.MenuItemView methodsFor: 'initialization'!

category: aCategory label: aLabel selector: aSelector receiver: aReceiver
    category := aCategory.
    label := aLabel.
    selector := aSelector.
    receiver := aReceiver.
!

buildWidgets
    super buildWidgets.
    wgt := (GtkMenuItem newWithLabel: label)
		on: 'activate' send: selector to: receiver;
		show;
		yourself.
!
!

GTK.GUI.MenuItemView methodsFor: 'accessing'!

external 
    ^wgt
!

category
    ^category
!
!
