"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

Primitive subclass: #PScrollBox
	instanceVariableNames: 'box horizontal vport prefSize allocSize btns shadow'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GUI-Components'!

PScrollBox subclass: #PHScrollBox
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GUI-Components'!

PScrollBox subclass: #PVScrollBox
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GUI-Components'!

!PScrollBox methodsFor: 'initialization'!

buildWidgets
    | adjs arrows |
    
	super buildWidgets.

	horizontal 
		ifTrue: 
			[ box := GtkHBox new: false spacing: 0.
			arrows := Array with: Gtk gtkArrowLeft with: Gtk gtkArrowRight. ]
		ifFalse: 
			[ box := GtkVBox new: false spacing: 0.
			arrows := Array with: Gtk gtkArrowUp with: Gtk gtkArrowDown. ].
	box show.

    btns := Array new: 2.
    1 to: 2 do: 
		[ :i | | btn al ar |
		(btn := GtkButton new) show.
		btns at: i put: btn.

		(al := GtkAlignment new: 0.5 yalign: 0.5 xscale: 0.0 yscale: 0.0) show.
		btn add: al.
		
		(ar := GtkArrow new: (arrows at: i)	shadowType: Gtk gtkShadowNone) show.
		al add: ar.

		btn on: 'clicked' send: #scrollClick:delta: to: self with: ((2 * i) - 3). ].

    (vport := GtkViewport new: nil vadjustment: nil) show.
	shadow ifNotNil: [ :r | vport setShadowType: shadow ].
    vport 
		on: 'size-request' send: #sizeRequest:requisition: to: self;
		on: 'size-allocate' send: #sizeAllocate:allocation: to: self.

    box 
		packStart: (btns at: 1) expand: false fill: false padding: 0;
		packStart: vport expand: true fill: true padding: 0;
		packEnd: (btns at: 2) expand: false fill: false padding: 0.	
!

attachViews
	childViews isNil ifTrue: [ ^self ].
	childViews do: [ :each | vport add: each external ].
!
!


!PScrollBox methodsFor: 'accessing'!

horizontal
	^horizontal
!

horizontal: aValue
    horizontal := aValue.
!

add: aChild
    super add: aChild.
    vport add: aChild external
!

external
    ^box
!

hide
	"FIXME"
	box hideAll
!

show
	"FIXME"
	box showAll
!

shadow
	^shadow
!

shadow: aShadowStyle
	shadow := aShadowStyle.
	vport setShadowType: shadow.
!
!

!PScrollBox methodsFor: 'events'!

scrollClick: aSender delta: aDelta
    | adj val |
    adj := vport getHadjustment.
    val := adj getValue + (aDelta * allocSize / 2).

    val := val max: 0.
    val := val min: (prefSize - allocSize - 4). "Not sure where 4 adjustment comes from"

    adj setValue: val.
!

sizeAllocate: aSender allocation: aAlloc
    | all |
    all := aAlloc.
    aAlloc class = CObject ifTrue: 
	[ [ all := aAlloc castTo: GtkAllocation ]
	    on: Error do: 
		[ :e | 'Cast to GtkAllocation failed'.
		^self ] ].
    allocSize := all width value.
    (prefSize isNil or: [ allocSize >= prefSize ])
		ifTrue: [ btns do: [ :each | each hide ] ]
		ifFalse: [ btns do: [ :each | each show ] ].
!

sizeRequest: aSender requisition: aReq
    horizontal 
		ifTrue: 
			[ prefSize := aReq width value.
			aReq width value: 35. ]
		ifFalse: 
			[ prefSize := aReq height value.
			aReq height value: 35. ].
! 
!

!PHScrollBox methodsFor: 'initialization'!

initialize
	super initialize.
	horizontal := true.
!
!

!PVScrollBox methodsFor: 'initialization'!

initialize
	super initialize.
	horizontal := false.
!
!
