"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

Primitive subclass: #PTree
	instanceVariableNames: 'frame tree treeView nodesMsg colsMsg changing events selection entries sorted '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GUI-Primitives'!

!PTree class methodsFor: 'instance creation'!

newWithTitles: aTitles
    "Convenience method for creating simple tree views"
	^(self new)
		buildColumns: aTitles.
!
!

!PTree methodsFor: 'accessing'!

tree
    ^tree
!

treeView
    ^treeView
!

colsMsg: aSelector
    colsMsg := aSelector
!

defaultExpand
    | it oc | 
	"Expand all root nodes"
	
    it := GtkTreeIter type new.
    oc := tree class.
		
	(tree iterChildren: it parent: nil)
		ifFalse: [ ^self ].
		
	[ treeView expandToPath: (tree getPath: it).
	tree iterNext: it. ] 
		whileTrue.
!

external
	^frame
!

nodesMsg: aSelector
    nodesMsg := aSelector
!

nodesMsg: aNodesMsg colsMsg: aColsMsg
    nodesMsg := aNodesMsg.
    colsMsg := aColsMsg.
!

on: aEvent send: aSelector to: aReceiver
	events on: aEvent send: aSelector to: aReceiver.
!

on: aEvent update: aReceiver with: aBlock
	events on: aEvent update: aReceiver with: aBlock.
!

on: aEvent updateWith: aBlock
	events on: aEvent updateWith: aBlock.
!

selection
    ^selection
!

sorted
    ^sorted
!

sorted: aValue
    sorted := aValue.
    "FIXME - And rebuild tree model"
!

refresh
    super refresh.
    self rebuildTree.
!
!

!PTree methodsFor: 'private-convenience'!

selectIter: anIter
    treeView expandToPath: (tree getPath: anIter).
    treeView getSelection selectIter: anIter.
    treeView scrollToCell: anIter column: nil useAlign: false rowAlign: 0.0 colAlign: 0.0.
!

showIter: anIter
    treeView 
	expandToPath: (tree getPath: anIter);
	scrollToCell: anIter column: nil useAlign: false rowAlign: 0.0 colAlign: 0.0.
!

buildColumns: aTitles
    "Convenience method for creating simple tree views"
    | col rend typs |

    typs := (CArrayCType elementType: CIntType numberOfElements: aTitles size + 1) new.
    typs at: 0 put: (GValue typeFromName: 'gint').
    1 to: aTitles size do: 
		[ :i | typs at: i put: (GValue typeFromName: 'gchararray'). ].

    tree := GtkTreeStore newv: aTitles size + 1 types: typs.

    treeView setModel: tree.

    aTitles doWithIndex: 
		[ :ttl :i |
		col := GtkTreeViewColumn new.
		treeView insertColumn: col position: -1. "-1 => append"
		col setTitle: ttl.
		col packStart: (rend := GtkCellRendererText new) expand: true.
		col addAttribute: rend attribute: 'text' column: i. ].
!
!

!PTree methodsFor: 'events'!

rowActivatedCallback: aSender
    "aSender class printNl.
    aSender parent address printNl.
    aSender getParent address printNl.
    aSender parent address = aSender getParent address ifFalse: [ self error: 'parent not equal' ]. ""FIXME"
	
    (events named: #nodeActivated) activateWith: selection.
!

selectionChangedCallback: aSender
    | it i |
    changing notNil ifTrue: [ ^self ].
    changing := treeView.

    treeView getSelection getMode = Gtk gtkSelectionSingle ifFalse: [ ^self ].

    it := GtkTreeIter type new.
    selection := (treeView getSelection getSelected: nil iter: it)
		ifTrue: [ entries at: (tree getOop: it column: 0). ]
		ifFalse: [ nil ].

    changing := nil.
	(events named: #nodeSelected) activateWith: selection.
    ^self
!

rebuildTree
    tree clear.
    (nodesMsg isNil | colsMsg isNil) ifTrue: [ ^self ].
    "Build the tree"
    entries := Dictionary new. "entries is the reverse lookup"
    self rebuildNode: nil atIter: nil.
	self defaultExpand. "display method - move out?"
!

rebuildNode: aNode atIter: aParent
    | nds cols it n |
    nds := self nodesFor: aNode.
    nds isNil ifTrue: [ ^self ].

    cols := OrderedCollection new: nds size.
    nds do: 
		[ :nd | cols add: nd -> (self colsFor: nd) ].

    sorted ifTrue:
		[ "Sort by first column"
		cols := cols asSortedCollection: 
			[ :a :b | 
			(a value at: 1) <= (b value at: 1) ]. ].

    cols do: 
		[ :each | 
		n := entries size + 1.
		it := GtkTreeIter type new.
		tree append: it parent: aParent.
		tree setOop: it column: 0 value: n.
		each value doWithIndex: 
			[ :col :i | tree setOop: it column: i value: col ].
		entries at: n put: each key.
		self rebuildNode: each key atIter: it. ].
!
!

!PTree methodsFor: 'change update'!

colsFor: aEntry
    ^data perform: colsMsg with: aEntry
!

firstColFor: aEntry
     ^(self colsFor: aEntry) at: 1.
!

nodesFor: aNode
    ^data perform: nodesMsg with: aNode.
!

!PTree methodsFor: 'initialization'!

rebuildStateChange: aStateChange
	self stateChange: aStateChange updateWith: [ self rebuildTree ].
!

setup: aData stateChange: aStateChange
	data := aData.
	self rebuildStateChange: aStateChange.
!

buildWidgets
    | scroll |  

    super buildWidgets.

    (frame := GtkFrame new: nil) show.

    (scroll := GtkScrolledWindow new: nil vadjustment: nil) show.
    frame add: scroll.
    scroll setPolicy: Gtk gtkPolicyAutomatic vscrollbarPolicy: Gtk gtkPolicyAutomatic.

    (treeView := GtkTreeView new) show.
    scroll add: treeView.

    treeView setHeadersVisible: false.
    treeView getSelection on: 'changed' send: #selectionChangedCallback: to: self.
    treeView on: 'row-activated' send: #rowActivatedCallback: to: self.

    "Now the subclass has to build the columns. Column 0 should always
     be gint - it will contain an id used to lookup the entry for the
     current selection."
!

initialize
    sorted := true.
    "listMode := false.
    inheritedMode := false."

    events := (NamedEventsSource new) initialize; yourself.
    events 
		add: #nodeActivated;
		add: #nodeSelected.
	
	super initialize.
!
!

!PTree methodsFor: 'private'!

tree: aTreeModel
    tree := aTreeModel.
    treeView setModel: tree.
!
!
