"======================================================================
|
|   Smalltalk GUI base class for widget wrappers with publish/subscribe
|
|
 ======================================================================"


"======================================================================
|
| Copyright 1992,94,95,99,2000,2001,2002 Free Software Foundation, Inc.
| Written by Brad Diller.
|
| This file is part of GNU Smalltalk.
|
| GNU Smalltalk is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| GNU Smalltalk; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

Namespace current: GTK.GUI!

GTK.GUI.View subclass:  #Primitive
	instanceVariableNames: 'menu dirty stateChangeMsg messageDispatch '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GUI-Framework'!

Primitive comment: 
nil!

!Primitive class methodsFor: 'displaying'!

updateViews
    "Update all the primitive views"
    rootViews do: [:rv | rv allPrimitivesDo: [:view | view update]].
!
!

!Primitive class methodsFor: 'initialization'!

new
    ^(super new)
		initialize;
		buildSubsidiaryViews;
		buildWidgets;
		attachViews;
		dirty: false;
		yourself.
!

update: aspect
	aspect == #returnFromSnapshot
		ifTrue: 
			["Build all the widgets first"
			self allPrimitivesDo: 
				[ :p | p buildWidgets ].
			"Now get on with the rest"
			self allPrimitivesDo: 
				[ :p | 
				p 
					attachViews;
					restoreNonStateProperties;
					resetState.	].
			self updateViews. 
			self changed: #returnFromSnapshot. ].
!
!

GTK.Gtk addDependent: Primitive!

!Primitive methodsFor: 'accessing'!

data: aData 
    data := aData!

dirty: aBoolean 
    dirty := aBoolean!

isDirty
    ^dirty!

menu
    ^menu
!
!

!Primitive methodsFor: 'initialization'!

initialize
	"Template method - initialize inst vars."
!

buildSubsidiaryViews
	"Template method - create any views that are fundamentally
	part of this one."
!

buildWidgets
	"Template method - create the widgets that make up this view."
!

attachViews
	"Template method - add the top-level widgets of all of the
	child views to this widget to their respective containers in
	this widget."
!

restoreNonStateProperties
	"Template method - set/restore any properties of this view's
	widgets that are not determined by its state (eg. window 
	position)."
!
!

!Primitive methodsFor: 'change management'!

allPrimitivesDo: aBlock 
	aBlock value: self.
	super allPrimitivesDo: aBlock.
!

collectPrimitives: aCollection 
	aCollection add: self.
	super collectPrimitives: aCollection.
!
!

!Primitive methodsFor: 'blue button menu items'!

close
    ^self rootView close! !

!Primitive methodsFor: 'displaying'!

resetState
	"self.getViewState == messageDispatch"
	messageDispatch isNil ifTrue: [ ^self ].
	messageDispatch do: 
		[ :vs | vs reset ].
!

display
    "Overridden in subclasses.  This method is used to support change/update 
     mechanism.  In the normal case, this method redraws entire view"

    ^self subclassResponsibility!

getViewState
    ^messageDispatch!

update: stateChanges 
    "Update object based on stateChanges"
    stateChanges do: 
	    [:sc | | viewState |
	    viewState := messageDispatch at: sc state.
	    viewState updateTo: sc counter]!

update
    | stateChanges |
    "Send a getStateChanges: currentViewState message to data object to compute state
     changes. Send a update: stateChanges message to self to update object"

    data isNil ifTrue: [^self].
	stateChanges := data getStateChanges: self getViewState.
    stateChanges notNil ifTrue: [self update: stateChanges].
!
!

!Primitive methodsFor: 'initialize-delete'!

handleUserChange: changeSelector 
    "This is used to update the data object in response to a user
     modification of the view"

    stateChangeMsg := changeSelector!

menuInit: theMenu 
    "The popup menu, theMenu, is stored in menu"

    menu := theMenu!

stateChange: theStateChange updateWith: block 
    messageDispatch isNil ifTrue: [messageDispatch := LookupTable new].
    messageDispatch at: theStateChange
	put: (GuiState 
		state: theStateChange
		counter: 0
		action: block)
!
!

!Primitive methodsFor: 'childViews and parentViews'!

windowView
	^parentView ifNotNil: 
		[ :pv | pv windowView ].
!
!