"======================================================================
|
|   Smalltalk GUI base class for widget wrappers with publish/subscribe
|
|
 ======================================================================"


"======================================================================
|
| Copyright 1992,94,95,99,2000,2001,2002 Free Software Foundation, Inc.
| Written by Brad Diller.
|
| This file is part of GNU Smalltalk.
|
| GNU Smalltalk is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| GNU Smalltalk; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

Namespace current: GTK.GUI!

GTK.GUI.Gui subclass:  #View
	instanceVariableNames: 'data parentView childViews events'
	classVariableNames: 'rootViews'
	poolDictionaries: ''
	category: 'GUI-Framework'!

View comment: 
nil!

!View class methodsFor: 'initialization'!

initializeRootViews
	rootViews := Set new.
!
!

View initializeRootViews!


!View class methodsFor: 'instance creation'!

new
	| r |
	r := self basicNew.
	rootViews add: r.
	^r.
!
!

!View class methodsFor: 'change management'!
	
allPrimitivesDo: aBlock 
	rootViews do: [:view | view allPrimitivesDo: aBlock].
!
!

!View methodsFor: 'accessing'!

data
    "Return view's data object"

    ^data! !

!View methodsFor: 'change management'!

allPrimitivesDo: aBlock 
    childViews notNil 
		ifTrue: [childViews do: [:view | view allPrimitivesDo: aBlock]].
!

canChangeState
    self rootView 
		allPrimitivesDo: [:view | view == self ifFalse: [view canUpdate ifFalse: [^false]]].
    ^true
!

canUpdate
    "Default is to return true"

    ^true
!

collectPrimitives: aCollection 
    childViews notNil 
		ifTrue: [childViews do: [:view | view collectPrimitives: aCollection]].
!
!

!View methodsFor: 'childViews and parentViews'!

add: aChildView 
    childViews isNil 
		ifTrue: [ childViews := OrderedCollection new ].
    childViews add: aChildView.
    aChildView parentView: self
!

remove: aChildView 
	aChildView parentView == self ifFalse: [ ^self ].
    childViews notNil 
		ifTrue: 
			[childViews remove: aChildView.
			aChildView parentView: nil ].
!

childViews
    "Return the view's collection of childViews"

    ^childViews!

parentView
    ^parentView!

parentView: aView 
    parentView := aView.
	aView isNil 
		ifTrue: [ rootViews add: self ]
		ifFalse: [ rootViews remove: self ifAbsent: [] ].
!

rootView
    ^parentView isNil ifTrue: [self] ifFalse: [parentView rootView]
!
!

!View methodsFor: 'display'!

beep
    "Beep once -- usually called when some user error is detected"

    "FIXME Gtk beep"
!
!

!View methodsFor: 'initialize-delete'!

remove
    data := nil.
    childViews isNil ifFalse: [childViews do: [:view | view remove]].
    parentView := childViews := nil
!
!

!View methodsFor: 'events'!

eventsMustBeAvailable
	events isNil ifTrue: [ self error: 'Does not have any events' ].
!

events
	events isNil ifTrue: 
		[ events := NamedEventsSource new ].
	^events
!

addEvent: aEvent
	self events add: aEvent.
!

on: aEvent send: aSelector to: aReceiver with: aData
	self eventsMustBeAvailable.
	events on: aEvent send: aSelector to: aReceiver with: aData.
!

on: aEvent send: aSelector to: aReceiver
	self eventsMustBeAvailable.
	events on: aEvent send: aSelector to: aReceiver.
!

on: aEvent update: aReceiver with: aBlock
	self eventsMustBeAvailable.
	events on: aEvent update: aReceiver with: aBlock.
!

on: aEvent updateWith: aBlock
	self eventsMustBeAvailable.
	events on: aEvent updateWith: aBlock.
!
!
