"======================================================================
|
|   Smalltalk GUI base class for widget wrappers with publish/subscribe
|
|
 ======================================================================"

"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

Namespace current: GTK.GUI!

GTK.GUI.Primitive subclass: #WindowedView
	instanceVariableNames: 'model popupMenuItems'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GUI-Framework'!

!GTK.GUI.WindowedView methodsFor: 'initialization'!

buildWidgets
    super buildWidgets.
!
!

!GTK.GUI.WindowedView methodsFor: 'popup menus'!

hookupPopupMenuTo: aWidget
    aWidget isNil ifTrue: [ ^self ].
    aWidget on: 'button-press-event' send: #mouseButtonPressCallback:event: to: self .
!

mouseButtonPressCallback: aWidget event: aGdkEvent
    | btn prev |
    "Cast aGdkEvent from GdkEvent to GdkEventButton"
    aGdkEvent changeClassTo: GdkEventButton.
    btn := aGdkEvent button value.
    btn = 3 ifFalse: [ ^false ].	 
	self windowView ifNotNil: 
		[ :wnd |
		wnd
			popupMenu: [ self popupMenuItemsFor: aWidget ]
			widget: aWidget
			button: aGdkEvent button value
			time: aGdkEvent time value ].
	^true.
!

addPopupMenuItems: aMenuItemList
	"Add context menu items that are independent of which 
	 widget was clicked on. Actually specify them in buildPopupMenuItems,
	 to save recreating the widgets all the time."
    popupMenuItems isNil ifTrue: 
		[ self buildPopupMenuItems ].
    aMenuItemList addAll: popupMenuItems.
    ^aMenuItemList.
!

addPopupMenuItems: aMenuItemList forWidget: aWidget
	"Add context menu items that apply specifically to aWidget.
	Create them as needed."
	
    ^aMenuItemList.
!

popupMenuItemsFor: aWidget
    | items pmnu pv |
    "Assemble the items for this menu"
    items := MenuItemList new.
	pv := self.
	[ pv notNil ] whileTrue: 
		[ pv addPopupMenuItems: items forWidget: aWidget.
		pv addPopupMenuItems: items.
		pv := pv parentView. ].
	^items
!

buildPopupMenuItems
    popupMenuItems := MenuItemList new.
    ^popupMenuItems.
!

!GTK.GUI.WindowedView methodsFor: 'menu callbacks'!

promptFileIn
    | dlg |
    dlg := FileSelection new: 'File in'.
    (dlg run = Gtk gtkResponseOk) 
		ifTrue: [ FileStream fileIn: dlg filename ].
    dlg destroy.
!

fileOutChangesCallback
	CurrentChanges 
		fileOutChangesTo: (self nextChangesFileEnding: '.st');
		clear.
!

promptFileOutChanges
    | pt nm |
    pt := Prompter 
		title: 'File out changes' 
		entry: 'File name:'
		buttons: (Prompter buttonsOkCancel).
	pt entryText: (self nextChangesFileEnding: '.st').

    (pt run = Gtk gtkResponseOk) ifTrue: 
		[ | nm |
		CurrentChanges fileOutChangesTo: pt entryText. ].
    pt destroy.
!

fileOutClassesCallback
    | wd |
    wd := Directory working.
    Directory working: (Smalltalk getenv: 'BROWSERDIR').
    CurrentClasses fileOutTo: (self nextChangesFileEnding: '_cls.st').
    Directory working: wd.
!

nextChangesFileEnding: anEnd
    | dir fn |
    dir := (Directory name: 'changes').
    dir exists 
	ifFalse: [
	    "Broken"
	    Directory create: (Directory working, '/changes').
	    fn := 1.
	]
	ifTrue: [  
	    fn := 0.
	    dir filesMatching: ('*', anEnd) do: [ :each | | n nm |
		n := each stripPath asInteger.
		(n > fn) ifTrue: [ fn := n ].
	    ].
	    fn := fn + 1.
	].
    ^'changes/', fn printString, anEnd.!

snapshotCallback
    | fn |
    fn := self nextChangesFileEnding: '.im'.
    ObjectMemory snapshot: fn.
    Transcript 
		print: 'Snapshot saved to ', fn; 
		cr.
!

quitCallback
    | dlg rs wnd |
	wnd := self windowView.
	wnd isNil ifTrue: [ ^self ].
    dlg := GtkMessageDialog 
		new: wnd external
		flags: Gtk gtkDialogModal 
		type: Gtk gtkMessageQuestion
		buttons: Gtk gtkButtonsYesNo
		message: 'Really quit?'.
    rs := dlg run.
    dlg destroy.
    (rs = Gtk gtkResponseYes)
		ifTrue: [ wnd closeAll. ].
! !

!GTK.GUI.WindowedView methodsFor: 'testing'!

isWindow
    ^false
! !

!GTK.GUI.WindowedView methodsFor: 'childViews and parentViews'!

windowView
	^parentView 
		ifNotNil: [ :p | p windowView ].
!
!
